"""An SQL Object to store information about our podcast library. """

from sqlobject import *
import feedparser

class PodcastDatabase(SQLObject):
    """A database of podcast names and urls."""
    name = StringCol()
    url = StringCol()

    def get_podcast_urls(self):
        """
        Get a string array of all the podcast URLs.

        Returns
        -------
        string_array
            A string array of all the podcast urls
        """
        podcasts = list(self.select())
        podcast_urls = []

        for podcast in podcasts:
            podcast_urls.append(podcast.url)

        return podcast_urls

    def get_podcast_names(self):
        """
        Get all the podcast names from the database.

        Returns
        -------
        string_array
            A string array of all the podcast names
        """
        podcasts = list(self.select())
        podcast_names = []

        for podcast in podcasts:
            podcast_names.append(podcast.name)

        return podcast_names

    def add_podcast(self, podcast_url):
        """
        Add a podcast to the databaseGet a string array of all the podcast URLs.

        Parameters
        ----------
        podcast_url : string
            The URL that you want to add to the database. The name is retreived
            from the feed title.
        """
        feed = feedparser.parse(podcast_url)
        name=feed.feed.title
        url=podcast_url

        check_db = self.select (self.q.name == name).count()
        if check_db == 0:
            PodcastDatabase(name=name,url=url)



"""An Object to store and retrive user settings"""

import os, configparser

class UserSettings(object):
    def __init__(self):
        """
        To initialise we create a config parser object with the default 
        player and save it to a file if it does not exist
        """
        self.basedir = "~/.podcast"

        self.user_config_dir = os.path.expanduser(self.basedir);
        self.user_config_path = self.user_config_dir + "/podcast-player.ini"

        self.config = configparser.ConfigParser()
        self.config.add_section('podcast')
        self.config['podcast']['player'] = "mplayer"

        if not os.path.isfile(self.user_config_path):
            with open(self.user_config_path, 'w') as f:
                self.config.write(f)

        self.config = configparser.ConfigParser()
        self.config.sections()
        self.config.read(self.user_config_path)

    def get_media_player(self):
        """
        Get the media player name

        Returns
        -------
        string
            the name of the media player of choice
        """
        return self.config['podcast']['player']

    def set_media_player(self, player_name):
        """
        Set the name of the media player of choice.

        Parameters
        ----------
        player_name : string
            The name of the media player of choice. mpv or mplayer
        """
        self.config.set('podcast','player',player_name)
        with open(self.user_config_path, 'w') as configfile:
            self.config.write(configfile)



"""
podcast

Usage:
  podcast
  podcast list
  podcast add <url>
  podcast set-player <player>
  podcast -h | --help
  podcast --version

Options:
  -h --help                         Show this screen.
  --version                         Show version.

Examples:
  podcast
  podcast list
  podcast set-player mpv
  podcast set-player mplayer
  podcast add https://my-podcast-url.com/feed.rss

Help:
  For help using this tool, please open an issue on the Github repository:
  https://github.com/aziezahmed/podcast-manager
"""
import os
import sys
import feedparser

from docopt import docopt
from os.path import expanduser
from sqlobject import *

from . import __version__ as VERSION
from . import PodcastDatabase
from . import UserSettings

def list_podcasts():
    """
    List the names of all the subscribed podcasts.
    """
    podcasts = PodcastDatabase.get_podcast_names(PodcastDatabase)
    os.system('clear')
    for podcast in podcasts:
        print(podcast)

def add_podcast(url):
    """
    List the names of all the subscribed podcasts.

    Parameters
    ----------
    url : string
        The URL of the podcast to subscribe to.
    """
    PodcastDatabase.add_podcast(PodcastDatabase, url)

def set_player(player):
    user_settings = UserSettings()
    user_settings.set_media_player(player)

def play_podcast(url):
    user_settings = UserSettings()
    player = user_settings.get_media_player()

    os.system('clear')
    os.system(player + " "+ url)
    

def episode_menu(podcast_choice):
    podcast_urls = PodcastDatabase.get_podcast_urls(PodcastDatabase)
    feed = feedparser.parse(podcast_urls[int(podcast_choice)-1])
    feed.entries.reverse()
    
    choice = ""
    while choice != "b":
        os.system('clear')
        for index, entry in enumerate(feed.entries):
            print(str(index+1) + " - " + entry['title'])
        print("b - Back")
        print("q - Quit")
        choice = input(">>  ")
        if choice.lower() == "q":
            sys.exit(0)
        elif choice.lower() != "b":
            url = feed.entries[int(choice)-1]["link"]
            play_podcast(url)
    podcast_menu()
    

def podcast_menu():
    podcast_names = PodcastDatabase.get_podcast_names(PodcastDatabase)
    os.system('clear')
    for index, name in enumerate(podcast_names):
        print(str(index+1) + " - " + name)
    print("q - Quit")
    choice = input(">>  ")
    if choice.lower() == "q":
        sys.exit(0)
    else:
        episode_menu(choice)


def main():
    """
    The main function. We will establish a connnection to the database and
    process the user's command.
    """

    basedir = "~/.podcast"
    path = basedir + os.sep + 'podcast.sqlite'

    # If the ~/.podcast directory does not exist, let's create it.
    if not os.path.exists(expanduser(basedir)):
        print("Creating base dir %s"%basedir)
        os.makedirs(expanduser(basedir))

    # Make a connection to the DB. Create it if it does not exist
    PodcastDatabase._connection = sqlite.builder()(expanduser(path), debug=False)
    PodcastDatabase.createTable(ifNotExists=True)

    # Run the docopt
    options = docopt(__doc__, version=VERSION)

    if(options["list"]):
        list_podcasts()

    elif(options["add"]):
        add_podcast(options["<url>"])

    elif(options["set-player"]):
        set_player(options["<player>"])

    else:
        podcast_menu()
